<?php
namespace Pluie\Zic\Ctrl;

trait AdminTrait
{
    /**
     * @protected
     * @param       string      $data
     * @return      string
     */
    protected function getHash($data)
    {
        return hash('sha256', $this->app['config']['security']['prefix_id'] . $data);
    }

    /**
     * @protected
     */
    protected function NewCsRfToken()
    {
        $this->app['session']->set('csrf', hash('sha256', $this->getHash(microtime(true))));
    }

    /**
     * @protected
     * @return      string
     */
    private function getCsRfToken()
    {
        return $this->app['session']->get('csrf');
    }

    /**
     * @protected
     * @return      bool
     */
    protected function isConnected()
    {
        $connected = $this->app['session']->get('authenticated');
        $this->app['log']('main')->debug(__line__ . ':' . __method__, compact('connected'));
        return $connected == true;
    }

    /**
     * @protected
     * @param       string      $msg
     */
    protected function setErrMsg($msg)
    {
        $this->app['session']->set('errorMsg', $msg);
    }

    /**
     * @protected
     * @param       bool    $keep
     * @return      string
     */
    protected function getErrMsg($keep=false)
    {
        $msg = $this->app['session']->get('errorMsg');
        if (!$keep) {
            $this->flushErrMsg();
        }
        return $msg;
    }

    /**
     * @protected
     */
    protected function flushErrMsg()
    {
        $this->app['session']->set('errorMsg', '');
    }

    /**
     * @protected
     * @return      array
     */
    protected function getAdminData()
    {
        $this->app['log']('main')->debug(__line__);
        $page    = $this->query("p", $this->isConnected() ? 'index' : 'connect');
        $this->app['log']('main')->debug(__line__, compact('page'));
        if ((empty($page) || $page == 'connect') && $this->isConnected()) {
            $this->app['log']('main')->debug(__line__);
            $page = 'index';
        }
        $this->app['log']('main')->debug(__line__, compact('page'));
        if (!$this->isConnected()) {
            $this->app['log']('main')->debug(__line__);
            $page = 'connect';
        }
        $this->app['log']('main')->debug(__line__, compact('page'));
        $section = $this->query("s", '0');
        $this->app['log']('main')->debug(__line__, compact('section'));
        $errMsg  = $this->getErrMsg();
        $tpl     = $this->app['tpl']('admin');
        $tplName = "admin/none.twig";
        $data    = [];
        switch($page) {
            case "index" :
                $this->app['log']('main')->debug(__line__);
                $tplName = 'admin/index.twig';
                $data = compact('page', 'section', 'errMsg');
                break;
            case "qrcode" :
                $this->app['log']('main')->debug(__line__);
                $tplName = 'admin/qrcode.twig';
                $data    = compact('page', 'section', 'errMsg');
                $this->manageQRCodes($data);
                break;
            case "connect" :
                $this->app['log']('main')->debug(__line__);
                $this->NewCsRfToken();
                $tplName = 'admin/login.twig';
                $csrf    = $this->getCsRfToken();
                $data = compact('page', 'csrf', 'errMsg');
                break;
        }
        $this->app['log']('main')->debug(__line__);
        $adminData['content'] = $tpl->render($tplName, compact('data'));
        return $adminData;
    }

    protected function getQRCodeHash($data)
    {
        return hash($this->app['config']['qrcode']['algo'], $this->app['config']['qrcode']['salt'] . $data);
    }

    protected function manageQRCodes(&$data)
    {
        if ($_SERVER['REQUEST_METHOD'] == "POST") {
            if ($data['section'] == 1) {
                $eventName   = $this->param('eventName');
                $url         = $_SERVER['REQUEST_SCHEME'] . '://' . $_SERVER['HTTP_HOST'] . "/access/$eventName/".$this->getQRCodeHash($eventName);
                $output      = null;
                $retval      = null;
                $fileNamePng = $this->app['path'] . "qrcodes/$eventName.png";
                $check       = isset($_FILES["imgPath"]["tmp_name"])
                             && !empty($_FILES["imgPath"]["tmp_name"])
                             && file_exists($_FILES["imgPath"]["tmp_name"]) 
                             && getimagesize($_FILES["imgPath"]["tmp_name"]);
                $addFile     = "";
                if ($check !== false) {
                    $addFile .= " -m \"{$_FILES["imgPath"]["tmp_name"]}\" ";
                }
                $cmd  = $this->app['config']['qrcode']['bin'] . " $addFile -o $fileNamePng \"$url\"";
                $data['sendEvent'] = $cmd;
                exec($cmd, $output, $retval);
                if ($retval == 0) {
                    $cmd = $this->app['config']['qrcode']['xtract'] . " -q --raw \"$fileNamePng\"";
                    exec($cmd, $output, $retval);
                    if ($retval == 0) {
                        $data['b64file'] = base64_encode(file_get_contents($fileNamePng));
                        $data['qrlink']  = $output[0];
                    }
                }
                $data += compact('output', 'retval');
                //~ $this->app['config']['qrcode']['bin'] . 
            }
        }
        if ($data['section'] == 2) {
            $action = $this->query('a'  , 'none');
            $evt    = $this->query('evt', '');
            $dir    = $this->app['path'] . "qrcodes/";
            $list   = @scandir($dir);
            $data['qrcodes'] = [];
            if (!empty($list)) {
                foreach($list as $entry) {
                    if (in_array($entry, ['.', '..'])) continue;
                    $event = substr($entry, 0, -4);
                    if ($action == "del" && $event == $evt) {
                        error_log("delete $evt");
                        unlink($dir . $entry);
                        continue;
                    }
                    if ($action != "del" || $event != $evt) {
                        $data['qrcodes'][$event] = base64_encode(file_get_contents($dir . $entry));
                    }
                }
            }
        }
        error_log(" < end qrcode method");
    }

    /**
     * @public
     * @return      Symfony\Component\HttpFoundation\Response
     */
    public function logout()
    {
        $this->app['session']->set('authenticated', false);
        return $this->app->redirect('/');
    }

    /**
     * @public
     * @return      Symfony\Component\HttpFoundation\Response
     */
    public function login()
    {
        $token = $this->getCsRfToken();
        $params = $this->params(['csrf', 'login', 'password']);
        if ($token == $params['csrf']) {
            
            $auth = $this->app['config']['site.auth'];
            
            $this->app['log']('main')->debug(' auth    : *'.var_export($auth, true).'*'); 
            $this->app['log']('main')->debug(' isset user : *'.var_export(isset($auth['marc']), true).'*'); 
            $this->app['log']('main')->debug(' isset login : *'.var_export(isset($auth[$params['login']]), true).'*'); 
            $this->app['log']('main')->debug(' login    : *'.$params['login'].'*'); 
            $this->app['log']('main')->debug(' hash     : *'.$this->getHash($params['password']).'*'); 
            $this->app['log']('main')->debug(' verify hash : '. var_export($this->getHash($params['password']) == $this->app['config']['site.auth'][$params['login']], true)); 
            if (isset($auth[$params['login']]) && $this->getHash($params['password']) == $this->app['config']['site.auth'][$params['login']]) {
                $this->app['log']('main')->debug("good is good");
                $this->setErrMsg("vous êtes connecté !");
                $this->app['session']->set('authenticated', true);
                $this->app['session']->set('login', $params['login']);
            } else {
                $this->app['log']('main')->debug("bad is bad");
                $this->setErrMsg("l'authentification à échouée !");
                $this->app['session']->set('authenticated', false);
            }
            sleep(2);
        }
        return $this->app->redirect('/admin');
    }

    public function checkAccess($event, $token)
    {
        $dir    = $this->app['path'] . "qrcodes/";
        $list   = @scandir($dir);
        //~ $this->app['session']->set('authenticated', false);
        if (!empty($list)) {
            foreach($list as $entry) {
                if (in_array($entry, ['.', '..'])) continue;
                error_log("entry $entry");
                if (substr($entry, 0, -4) == $event && $this->getQRCodeHash($event) == $token) {
                    $this->app['session']->set('premium', $event);
                }
            }
        }
        return $this->app->redirect('/');
    }

    public function readQRCodeData($event)
    {
        $done   = false;
        $dir    = $this->app['path'] . "qrcodes/";
        $list   = @scandir($dir);
        $data   = [];
        if (!empty($list)) {
            foreach($list as $entry) {
                if (in_array($entry, ['.', '..'])) continue;
                $evt = substr($entry, 0, -4);
                if ($event == $evt) {
                    $fileNamePng = $this->app['path'] . "qrcodes/$entry";
                    $cmd = $this->app['config']['qrcode']['xtract'] . " -q --raw \"$fileNamePng\"";
                    $output = null;
                    $retval = null;
                    exec($cmd, $output, $retval);
                    $done = $retval == 0;
                    $data = $output[0];
                    $this->app['log']('main')->debug(__method__, compact('cmd', 'retval', 'output'));
                    break;
                }
            }
        }
        return $this->app->json(compact('done', 'data'), 200);
    }

    /**
     * @public
     * @return      string
     */
    public function admin()
    {
        return $this->index(true);
    }

    /**
     * @protected
     * @param       Silex\ControllerCollection      &$index
     * @param       string                          $prefixMethod
     */
    protected function getAdminRoutes()
    {
        return [
            '/access/{event}/{token}'    => 'checkAccess',
            '/logout'                    => 'logout',
            '/admin'                     => 'admin',
            '/admin/login'               => 'login',
            '/admin/qrcode/{event}/read' => 'readQRCodeData'
        ];
    }
}
