<?php
namespace Pluie\Zic\Ctrl;

use Ma\Web\WebController;
use Pluie\Zic\Ctrl\FileTrait;
use Pluie\Zic\Ctrl\ZicTrait;
use Pluie\Zic\Ctrl\AdminTrait;

/**
 * @package         Pluie\Zic\Ctrl
 * @class           ZicCtrl
 * @extends         WebController
 * @author          Mahdi Abu Yasmine
 * @date            2024-02-16 21:09:01 CET
 * @copyright       meta-tech.academy
 */
class ZicCtrl extends WebController
{
    use FileTrait;
    use ZicTrait;
    use AdminTrait;

    /**
     * @methodindex
     * @public
     * @param       bool    $admin
     * @return      Symfony\Component\HttpFoundation\Response
     */
    public function index($admin=false)
    {
        $groupName     = $this->app['config']['group']['name'];
        $labelGroups   = $this->app['config']['group']['others'];
        $config        = $this->app['config'];
        $infos         = $config['info'];
        $group         = $config['group'];
        $nav           = $this->app['config']['site.nav']['list'];
        $navWidth      = $this->app['config']['site.nav']['width'];
        $dataHome      = $this->app['config']['site.home'];
        $dataBio       = $this->app['config']['site.bio'];
        $dataQuote     = $this->app['config']['site.quote'];
        $dataCopyright = $this->app['config']['site.copyright'];
        $dataDates     = $this->app['config']['site.dates'];
        $dataPhotos    = $this->app['config']['site.photos'];
        $dataContact   = $this->app['config']['site.contact'];
        $dataTech      = $this->app['config']['site.tech'];
        $dataProtect   = $this->app['config']['site.protect'];
        uasort($nav, function ($v1, $v2) {
            return $v1['order'] == $v2['order'] ? 0 : ($v1['order'] < $v2['order'] ? -1 : 1);
        });
        $albums      = $this->getAlbums('albums', $dataProtect);
        $videos      = $this->getAlbums('videos', $dataProtect);
        $params      = $this->getParams();
        $randQuotes  = range(1, count($dataQuote));
        shuffle($randQuotes);
        $jsonAlbums  = json_encode($albums);
        $jsonVideos  = json_encode($videos);
        $this->app['session']->set($groupName.'-albums', $jsonAlbums);
        $this->app['session']->set($groupName.'-videos', $jsonVideos);
        $adminData = [];
        if ($admin) {
            $adminData = $this->getAdminData();
        }
        $premium = $this->app['session']->get('premium');
        return $this->tpl->render(
            'page.twig', compact(
                'adminData'    , 'group'     , 'groupName' , 'infos' , 'dataTech'   , 'dataPhotos', 'dataContact', 'dataDates',
                'dataCopyright', 'dataBio'   , 'dataHome'  , 'nav'   , 'navWidth'   , 'randQuotes', 'dataQuote'  , 'premium'  ,
                'albums'       , 'videos'    , 'jsonAlbums', 'config', 'labelGroups', 'params'    , 'adminData'
            )
        );
    }

    public function route2($param1, $param2)
    {
        $this->app['session']->set('kako','zuru');
        $this->app['session']->set('kako','zuru');
        $sess = $_SESSION;
        $data = compact('param1', 'param2', 'sess');
        return $this->tpl->render('test.twig', compact('data'));
    }

    public function connect(): array
    {

        return $this->getAdminRoutes() + [
            "/"                                       => "index",
            "/file/albums/{indexAlbum}/{code}/{type}" => "getFileAlbums",
            "/file/videos/{indexAlbum}/{code}/{type}" => "getFileVideos",
            "/{param1}/{param2}/route2"               => "route2"
        ];
    }
}
