<?php
namespace Pluie\Zic\Ctrl;

trait ZicTrait
{
    /**
     * @protected
     * @return      array
     */
    protected function getParams()
    {
        $params = [];
        foreach(file($this->app['path.data'].'data-param.txt') as $l){
            if (preg_match("/([^=]*)=(.*)/", $l, $matches)) {
                $params[trim($matches[1])] = trim($matches[2]);
            }
        }
        return $params;
    }

    /**
     * @protected
     * @return      array
     */
    protected function checkAlbumProtection($type, $albumName, $premium, $protect)
    {
        $locked = false;
        if (isset($protect['locks'][$type])) {
            foreach($protect['locks'][$type] as $lockName) {
                if ($albumName == $lockName) {
                    $locked = true;
                }
            }
        }
        if (!empty($premium) && $locked) {
            if (isset($protect['events'][$premium])) {
                if (is_array($protect['events'][$premium])) {
                    if (!empty($protect['events'][$premium])) {
                        foreach($protect['events'][$premium] as $targetName) {
                            if ($albumName == $targetName) {
                                $locked = false;
                                break;
                            }
                        }
                    }
                } elseif (is_string($protect['events'][$premium])) {
                    if ($albumName == $protect['events'][$premium]) {
                        $locked = false;
                    }
                }
            }
        }
        return $locked;
    }

    /**
     * @protected
     * @return      array
     */
    protected function getAlbums($type='albums', $protect=[])
    {
        $albums  = [];
        if ($dir = opendir($this->app["path.$type"])){
            while(false !== ($f = readdir($dir))){
                if($f != '.' && $f != '..'){
                    $albums[] = $f;
                }
            }
            closedir($dir);
            sort($albums);
            $premium = $this->app['session']->get('premium');
            foreach($albums as $n => $d){
                $album = new \stdclass();
                $album->pos    = substr($d,0,2);
                $album->name   = substr($d,3);
                $album->tracks = [];
                $album->cover  = null;
                $album->type   = $type;
                $album->locked = $this->checkAlbumProtection($type, $album->name, $premium, $protect);
                if (is_dir($this->app["path.$type"].$d) && $subdir = opendir($this->app["path.$type"].$d)) {
                    foreach(['.jpg', '.png'] as $ext) {
                        if (file_exists($this->app['path.cover']. "$type/{$album->name}$ext")) {
                            $album->cover = "/img/cover/$type/{$album->name}$ext";
                            break;
                        }
                    }
                    while (false !== ($t = readdir($subdir))) {
                        if ($t != '.' && $t != '..' && (substr($t,-4)=='.ogg' || substr($t,-8)=='.youtube')) {
                            $album->tracks[] = $t;
                        }
                    }
                    closedir($subdir);
                    sort($album->tracks);
                    foreach($album->tracks as $i => $t){
                        $track = new \stdclass();
                        $track->pos   = substr($t, 0, 2);
                        $track->ftype = $album->type;
                        $track->type  = substr($t,-4)=='.ogg' ? 'sound' : 'video';
                        $track->name  = substr($t, 3, $track->type == "sound" ? -4 : -8);
                        $track->license = "Free Public Music License";
                        $license = $this->app['path.files'] . $d . '/' . $track->pos . '-' . $track->name . ".license";
                        if (file_exists($license)) {
                            $track->license = preg_replace('/[\s\r\n]$/', '', file_get_contents($license));
                        }
                        if ($track->type == "video") {
                            $track->url = $album->locked ? 'locked' : preg_replace('/[\s\r\n]$/', '', file_get_contents($this->app["path.$type"].$d.'/'.$t));
                        }
                        $track->album = $n;
                        $track->code  = hash('sha256', $this->app['config']['security']['prefix_id'] . date('His') . $track->name);
                        $album->tracks[$track->code . ($album->locked ? date('Hisis') . 'Kml#rJko!!' : '')] = $track;
                        unset($album->tracks[$i]);
                    }
                    $albums[$n] = $album;
                }
            }
        }
        return $albums;
    }
}
